# Generated by roxygen2: do not edit by hand

S3method(as.data.frame,sdbuildR_sim)
S3method(as.data.frame,sdbuildR_xmile)
S3method(plot,sdbuildR_ensemble)
S3method(plot,sdbuildR_sim)
S3method(plot,sdbuildR_xmile)
S3method(print,summary.sdbuildR_xmile)
S3method(summary,sdbuildR_xmile)
export("%REM%")
export(build)
export(clean_name)
export(contains_IM)
export(convert_u)
export(debugger)
export(drop_u)
export(ensemble)
export(expit)
export(export_plot)
export(find_dependencies)
export(get_build_code)
export(get_regex_time_units)
export(get_regex_units)
export(get_units)
export(has_internet)
export(header)
export(indexof)
export(insightmaker_to_sfm)
export(install_julia_env)
export(julia_status)
export(length_IM)
export(logistic)
export(logit)
export(macro)
export(mod)
export(model_units)
export(nonnegative)
export(pulse)
export(ramp)
export(rbool)
export(rdist)
export(rem)
export(round_IM)
export(saveat_func)
export(seasonal)
export(sigmoid)
export(sim_specs)
export(simulate)
export(solvers)
export(step)
export(u)
export(unit_prefixes)
export(url_to_IM)
export(use_julia)
export(use_threads)
export(xmile)
importFrom(rlang,.data)
importFrom(stringr,str_locate_all)
importFrom(stringr,str_sub)
