# meta() requires sfm argument [plain]

    Code
      meta(name = "Test")
    Condition
      Error in `missing_arg()`:
      ! Missing required argument `object`.

# meta() requires sfm argument [ansi]

    Code
      meta(name = "Test")
    Condition
      [1m[33mError[39m in `missing_arg()`:[22m
      [1m[22m[33m![39m Missing required argument `object`.

