% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/interpret_unit_test.R
\name{op_precedence}
\alias{op_precedence}
\title{Get the precedence level of an operator}
\usage{
op_precedence(op)
}
\description{
Higher number = binds tighter. Based on R's actual operator precedence.
}
\keyword{internal}
