% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{summary.simulate_stockflow}
\alias{summary.simulate_stockflow}
\title{Summarize simulation results}
\usage{
\method{summary}{simulate_stockflow}(object, ...)
}
\arguments{
\item{object}{A simulation result of class \code{\link[=simulate.stockflow]{simulate_stockflow}}}

\item{...}{Additional arguments (unused)}
}
\value{
A \code{data.frame} with columns \code{variable}, \code{min}, \code{mean}, \code{max}, \code{final}.
}
\description{
Returns a data frame with per-variable summary statistics (min, mean, max,
and final value) over the simulated time range.
}
\examples{
sfm <- stockflow("sir")
sim <- simulate(sfm)
summary(sim)

}
\seealso{
\code{\link[=print.simulate_stockflow]{print.simulate_stockflow()}}, \code{\link[=simulate.stockflow]{simulate.stockflow()}}
}
\concept{simulate}
