% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/sdbuildR_custom_func.R
\name{hill}
\alias{hill}
\title{Hill function}
\usage{
hill(x, slope = 1, midpoint = 0.5, upper = 1)
}
\arguments{
\item{x}{Value at which to evaluate the function}

\item{slope}{Slope of Hill function at the midpoint. Defaults to 1.}

\item{midpoint}{Midpoint of Hill function where the output is \code{upper/2}. Defaults to 0.5.}

\item{upper}{Upper asymptote (maximal value) of the Hill function. Defaults to 1.}
}
\value{
Numeric value given by \deqn{f(x) = \frac{upper \cdot x^{slope}}{midpoint^{slope} + x^{slope}}}
}
\description{
Computes the Hill function with configurable slope, midpoint, and upper asymptote.
}
\details{
The Hill function is a smooth S-shaped curve (when slope > 1) bounded between 0 and \code{upper}.
It transitions from near 0 to near \code{upper} around the \code{midpoint}, with the steepness
of this transition controlled by \code{slope}. See \url{https://en.wikipedia.org/wiki/Hill_equation_\%28biochemistry\%29} for more details.
}
\examples{
hill(0)

# Adjust parameters
hill(0, slope = 5, midpoint = 0.5, upper = 10)

}
\concept{convenience}
