% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/utils.R
\name{has_internet}
\alias{has_internet}
\title{Check if user has internet}
\usage{
has_internet()
}
\value{
Logical value
}
\description{
Internal function
}
\examples{
has_internet()

}
\concept{internal}
