% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/stockflow.R
\name{print.stockflow}
\alias{print.stockflow}
\title{Print overview of stock-and-flow model}
\usage{
\method{print}{stockflow}(x, ...)
}
\arguments{
\item{x}{A stock-and-flow model object of class \code{\link{stockflow}}}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}
}
\description{
Prints a descriptive overview of the model structure, including stock-flow
topology, variable names, and simulation settings. For model diagnostics,
use \code{\link[=summary.stockflow]{summary()}}.
}
\examples{
sfm <- stockflow("sir")
print(sfm)

}
\seealso{
\code{\link[=summary.stockflow]{summary.stockflow()}}, \code{\link[=dependencies]{dependencies()}}
}
\concept{build}
