% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/simulate.R
\name{print.compare_stockflow}
\alias{print.compare_stockflow}
\title{Print comparison of two stock-and-flow models}
\usage{
\method{print}{compare_stockflow}(x, ...)
}
\arguments{
\item{x}{An object of class \code{\link[=compare_models]{compare_stockflow}}}

\item{...}{Additional arguments (unused)}
}
\value{
Invisibly returns \code{x}.
}
\description{
Print comparison of two stock-and-flow models
}
\concept{build}
